@@ -23,6 +23,7 @@
 
 #include "wine/test.h"
 #include "msg.h"
+#include "v6util.h"
 
 #define expect(expected, got) ok(got == expected, "Expected %d, got %d\n", expected, got)
 #define NUM_MSG_SEQUENCE 2
@@ -439,15 +440,14 @@ static HWND create_parent_window(void){
 static LRESULT WINAPI trackbar_subclass_proc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam){
     WNDPROC oldproc = (WNDPROC)GetWindowLongPtrA(hwnd, GWLP_USERDATA);
     static LONG defwndproc_counter = 0;
+    struct message msg = { 0 };
     LRESULT ret;
-    struct message msg;
 
     msg.message = message;
     msg.flags = sent|wparam|lparam;
     if (defwndproc_counter) msg.flags |= defwinproc;
     msg.wParam = wParam;
     msg.lParam = lParam;
-    msg.id = 0;
     add_message(sequences, TRACKBAR_SEQ_INDEX, &msg);
 
     defwndproc_counter++;
@@ -1297,8 +1297,12 @@ static void test_create(void)
 
 START_TEST(trackbar)
 {
+    ULONG_PTR cookie;
+    HANDLE ctxt;
+
+    LoadLibraryA("comctl32.dll");
+
     init_msg_sequences(sequences, NUM_MSG_SEQUENCE);
-    InitCommonControls();
 
     /* create parent window */
     hWndParent = create_parent_window();
@@ -1325,5 +1329,25 @@ START_TEST(trackbar)
     test_ignore_selection();
     test_initial_state();
 
+    if (!load_v6_module(&cookie, &ctxt))
+        return;
+
+    test_trackbar_buddy();
+    test_line_size();
+    test_page_size();
+    test_position();
+    test_range();
+    test_selection();
+    test_thumb_length();
+    test_tic_settings();
+    test_tic_placement();
+    test_tool_tips();
+    test_unicode();
+    test_TBS_AUTOTICKS();
+    test_ignore_selection();
+    test_initial_state();
+
+    unload_v6_module(cookie, ctxt);
+
     DestroyWindow(hWndParent);
 }
