@@ -27,14 +27,15 @@
 #include "windef.h"
 #include "winbase.h"
 #include "winerror.h"
-#include "wine/debug.h"
 #include "msi.h"
 #include "msidefs.h"
 #include "objbase.h"
 #include "oleauto.h"
 
 #include "msipriv.h"
-#include "msiserver.h"
+#include "winemsi.h"
+#include "wine/heap.h"
+#include "wine/debug.h"
 #include "wine/unicode.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(msi);
@@ -74,38 +75,15 @@ UINT WINAPI MsiDoActionW( MSIHANDLE hInstall, LPCWSTR szAction )
     package = msihandle2msiinfo( hInstall, MSIHANDLETYPE_PACKAGE );
     if (!package)
     {
-        HRESULT hr;
-        BSTR action;
-        IWineMsiRemotePackage *remote_package;
+        MSIHANDLE remote;
 
-        remote_package = (IWineMsiRemotePackage *)msi_get_remote( hInstall );
-        if (!remote_package)
+        if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        action = SysAllocString( szAction );
-        if (!action)
-        {
-            IWineMsiRemotePackage_Release( remote_package );
-            return ERROR_OUTOFMEMORY;
-        }
-
-        hr = IWineMsiRemotePackage_DoAction( remote_package, action );
-
-        SysFreeString( action );
-        IWineMsiRemotePackage_Release( remote_package );
-
-        if (FAILED(hr))
-        {
-            if (HRESULT_FACILITY(hr) == FACILITY_WIN32)
-                return HRESULT_CODE(hr);
-
-            return ERROR_FUNCTION_FAILED;
-        }
-
-        return ERROR_SUCCESS;
+        return remote_DoAction(remote, szAction);
     }
  
-    ret = ACTION_PerformAction( package, szAction );
+    ret = ACTION_PerformAction( package, szAction, SCRIPT_NONE );
     msiobj_release( &package->hdr );
 
     return ret;
@@ -140,38 +118,18 @@ UINT WINAPI MsiSequenceW( MSIHANDLE hInstall, LPCWSTR szTable, INT iSequenceMode
 
     TRACE("%s, %d\n", debugstr_w(szTable), iSequenceMode);
 
+    if (!szTable)
+        return ERROR_INVALID_PARAMETER;
+
     package = msihandle2msiinfo( hInstall, MSIHANDLETYPE_PACKAGE );
     if (!package)
     {
-        HRESULT hr;
-        BSTR table;
-        IWineMsiRemotePackage *remote_package;
+        MSIHANDLE remote;
 
-        remote_package = (IWineMsiRemotePackage *)msi_get_remote( hInstall );
-        if (!remote_package)
+        if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        table = SysAllocString( szTable );
-        if (!table)
-        {
-            IWineMsiRemotePackage_Release( remote_package );
-            return ERROR_OUTOFMEMORY;
-        }
-
-        hr = IWineMsiRemotePackage_Sequence( remote_package, table, iSequenceMode );
-
-        SysFreeString( table );
-        IWineMsiRemotePackage_Release( remote_package );
-
-        if (FAILED(hr))
-        {
-            if (HRESULT_FACILITY(hr) == FACILITY_WIN32)
-                return HRESULT_CODE(hr);
-
-            return ERROR_FUNCTION_FAILED;
-        }
-
-        return ERROR_SUCCESS;
+        return remote_Sequence(remote, szTable, iSequenceMode);
     }
     ret = MSI_Sequence( package, szTable );
     msiobj_release( &package->hdr );
@@ -190,7 +148,11 @@ UINT msi_strcpy_to_awstring( const WCHAR *str, int len, awstring *awbuf, DWORD *
     if (len < 0) len = strlenW( str );
  
     if (awbuf->unicode && awbuf->str.w)
+    {
         memcpy( awbuf->str.w, str, min(len + 1, *sz) * sizeof(WCHAR) );
+        if (*sz && len >= *sz)
+            awbuf->str.w[*sz - 1] = 0;
+    }
     else
     {
         int lenA = WideCharToMultiByte( CP_ACP, 0, str, len + 1, NULL, 0, NULL, NULL );
@@ -239,55 +201,19 @@ static UINT MSI_GetTargetPath( MSIHANDLE hInstall, LPCWSTR szFolder,
     package = msihandle2msiinfo( hInstall, MSIHANDLETYPE_PACKAGE );
     if (!package)
     {
-        HRESULT hr;
-        IWineMsiRemotePackage *remote_package;
         LPWSTR value = NULL;
-        BSTR folder;
-        DWORD len;
+        MSIHANDLE remote;
 
-        remote_package = (IWineMsiRemotePackage *)msi_get_remote( hInstall );
-        if (!remote_package)
+        if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        folder = SysAllocString( szFolder );
-        if (!folder)
-        {
-            IWineMsiRemotePackage_Release( remote_package );
-            return ERROR_OUTOFMEMORY;
-        }
+        r = remote_GetTargetPath(remote, szFolder, &value);
+        if (r != ERROR_SUCCESS)
+            return r;
 
-        len = 0;
-        hr = IWineMsiRemotePackage_GetTargetPath( remote_package, folder, NULL, &len );
-        if (FAILED(hr))
-            goto done;
-
-        len++;
-        value = msi_alloc(len * sizeof(WCHAR));
-        if (!value)
-        {
-            r = ERROR_OUTOFMEMORY;
-            goto done;
-        }
-
-        hr = IWineMsiRemotePackage_GetTargetPath( remote_package, folder, value, &len );
-        if (FAILED(hr))
-            goto done;
-
-        r = msi_strcpy_to_awstring( value, len, szPathBuf, pcchPathBuf );
-
-done:
-        IWineMsiRemotePackage_Release( remote_package );
-        SysFreeString( folder );
-        msi_free( value );
-
-        if (FAILED(hr))
-        {
-            if (HRESULT_FACILITY(hr) == FACILITY_WIN32)
-                return HRESULT_CODE(hr);
-
-            return ERROR_FUNCTION_FAILED;
-        }
+        r = msi_strcpy_to_awstring(value, -1, szPathBuf, pcchPathBuf);
 
+        midl_user_free(value);
         return r;
     }
 
@@ -408,55 +334,19 @@ static UINT MSI_GetSourcePath( MSIHANDLE hInstall, LPCWSTR szFolder,
     package = msihandle2msiinfo( hInstall, MSIHANDLETYPE_PACKAGE );
     if (!package)
     {
-        HRESULT hr;
-        IWineMsiRemotePackage *remote_package;
         LPWSTR value = NULL;
-        BSTR folder;
-        DWORD len;
+        MSIHANDLE remote;
 
-        remote_package = (IWineMsiRemotePackage *)msi_get_remote( hInstall );
-        if (!remote_package)
+        if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        folder = SysAllocString( szFolder );
-        if (!folder)
-        {
-            IWineMsiRemotePackage_Release( remote_package );
-            return ERROR_OUTOFMEMORY;
-        }
+        r = remote_GetSourcePath(remote, szFolder, &value);
+        if (r != ERROR_SUCCESS)
+            return r;
 
-        len = 0;
-        hr = IWineMsiRemotePackage_GetSourcePath( remote_package, folder, NULL, &len );
-        if (FAILED(hr))
-            goto done;
-
-        len++;
-        value = msi_alloc(len * sizeof(WCHAR));
-        if (!value)
-        {
-            r = ERROR_OUTOFMEMORY;
-            goto done;
-        }
-
-        hr = IWineMsiRemotePackage_GetSourcePath( remote_package, folder, value, &len );
-        if (FAILED(hr))
-            goto done;
-
-        r = msi_strcpy_to_awstring( value, len, szPathBuf, pcchPathBuf );
-
-done:
-        IWineMsiRemotePackage_Release( remote_package );
-        SysFreeString( folder );
-        msi_free( value );
-
-        if (FAILED(hr))
-        {
-            if (HRESULT_FACILITY(hr) == FACILITY_WIN32)
-                return HRESULT_CODE(hr);
-
-            return ERROR_FUNCTION_FAILED;
-        }
+        r = msi_strcpy_to_awstring(value, -1, szPathBuf, pcchPathBuf);
 
+        midl_user_free(value);
         return r;
     }
 
@@ -614,39 +504,12 @@ UINT WINAPI MsiSetTargetPathW(MSIHANDLE hInstall, LPCWSTR szFolder,
     package = msihandle2msiinfo(hInstall, MSIHANDLETYPE_PACKAGE);
     if (!package)
     {
-        HRESULT hr;
-        BSTR folder, path;
-        IWineMsiRemotePackage *remote_package;
+        MSIHANDLE remote;
 
-        remote_package = (IWineMsiRemotePackage *)msi_get_remote( hInstall );
-        if (!remote_package)
+        if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        folder = SysAllocString( szFolder );
-        path = SysAllocString( szFolderPath );
-        if (!folder || !path)
-        {
-            SysFreeString(folder);
-            SysFreeString(path);
-            IWineMsiRemotePackage_Release( remote_package );
-            return ERROR_OUTOFMEMORY;
-        }
-
-        hr = IWineMsiRemotePackage_SetTargetPath( remote_package, folder, path );
-
-        SysFreeString(folder);
-        SysFreeString(path);
-        IWineMsiRemotePackage_Release( remote_package );
-
-        if (FAILED(hr))
-        {
-            if (HRESULT_FACILITY(hr) == FACILITY_WIN32)
-                return HRESULT_CODE(hr);
-
-            return ERROR_FUNCTION_FAILED;
-        }
-
-        return ERROR_SUCCESS;
+        return remote_SetTargetPath(remote, szFolder, szFolderPath);
     }
 
     ret = MSI_SetTargetPathW( package, szFolder, szFolderPath );
@@ -697,21 +560,12 @@ BOOL WINAPI MsiGetMode(MSIHANDLE hInstall, MSIRUNMODE iRunMode)
     package = msihandle2msiinfo(hInstall, MSIHANDLETYPE_PACKAGE);
     if (!package)
     {
-        BOOL ret;
-        HRESULT hr;
-        IWineMsiRemotePackage *remote_package;
+        MSIHANDLE remote;
 
-        remote_package = (IWineMsiRemotePackage *)msi_get_remote(hInstall);
-        if (!remote_package)
+        if (!(remote = msi_get_remote(hInstall)))
             return FALSE;
 
-        hr = IWineMsiRemotePackage_GetMode(remote_package, iRunMode, &ret);
-        IWineMsiRemotePackage_Release(remote_package);
-
-        if (hr == S_OK)
-            return ret;
-
-        return FALSE;
+        return remote_GetMode(remote, iRunMode);
     }
 
     switch (iRunMode)
@@ -736,15 +590,15 @@ BOOL WINAPI MsiGetMode(MSIHANDLE hInstall, MSIRUNMODE iRunMode)
         break;
 
     case MSIRUNMODE_SCHEDULED:
-        r = (package->script == SCRIPT_INSTALL);
+        r = package->scheduled_action_running;
         break;
 
     case MSIRUNMODE_ROLLBACK:
-        r = (package->script == SCRIPT_ROLLBACK);
+        r = package->rollback_action_running;
         break;
 
     case MSIRUNMODE_COMMIT:
-        r = (package->script == SCRIPT_COMMIT);
+        r = package->commit_action_running;
         break;
 
     case MSIRUNMODE_MAINTENANCE:
@@ -789,25 +643,12 @@ UINT WINAPI MsiSetMode(MSIHANDLE hInstall, MSIRUNMODE iRunMode, BOOL fState)
     package = msihandle2msiinfo( hInstall, MSIHANDLETYPE_PACKAGE );
     if (!package)
     {
-        HRESULT hr;
-        IWineMsiRemotePackage *remote_package;
+        MSIHANDLE remote;
 
-        remote_package = (IWineMsiRemotePackage *)msi_get_remote( hInstall );
-        if (!remote_package)
+        if (!(remote = msi_get_remote(hInstall)))
             return FALSE;
 
-        hr = IWineMsiRemotePackage_SetMode( remote_package, iRunMode, fState );
-        IWineMsiRemotePackage_Release( remote_package );
-
-        if (FAILED(hr))
-        {
-            if (HRESULT_FACILITY(hr) == FACILITY_WIN32)
-                return HRESULT_CODE(hr);
-
-            return ERROR_FUNCTION_FAILED;
-        }
-
-        return ERROR_SUCCESS;
+        return remote_SetMode(remote, iRunMode, fState);
     }
 
     switch (iRunMode)
@@ -844,9 +685,6 @@ UINT WINAPI MsiSetFeatureStateA(MSIHANDLE hInstall, LPCSTR szFeature,
 
     szwFeature = strdupAtoW(szFeature);
 
-    if (!szwFeature)
-        return ERROR_FUNCTION_FAILED;
-   
     rc = MsiSetFeatureStateW(hInstall,szwFeature, iState); 
 
     msi_free(szwFeature);
@@ -977,38 +815,18 @@ UINT WINAPI MsiSetFeatureStateW(MSIHANDLE hInstall, LPCWSTR szFeature,
 
     TRACE("%s %i\n",debugstr_w(szFeature), iState);
 
+    if (!szFeature)
+        return ERROR_UNKNOWN_FEATURE;
+
     package = msihandle2msiinfo(hInstall, MSIHANDLETYPE_PACKAGE);
     if (!package)
     {
-        HRESULT hr;
-        BSTR feature;
-        IWineMsiRemotePackage *remote_package;
+        MSIHANDLE remote;
 
-        remote_package = (IWineMsiRemotePackage *)msi_get_remote(hInstall);
-        if (!remote_package)
+        if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        feature = SysAllocString(szFeature);
-        if (!feature)
-        {
-            IWineMsiRemotePackage_Release(remote_package);
-            return ERROR_OUTOFMEMORY;
-        }
-
-        hr = IWineMsiRemotePackage_SetFeatureState(remote_package, feature, iState);
-
-        SysFreeString(feature);
-        IWineMsiRemotePackage_Release(remote_package);
-
-        if (FAILED(hr))
-        {
-            if (HRESULT_FACILITY(hr) == FACILITY_WIN32)
-                return HRESULT_CODE(hr);
-
-            return ERROR_FUNCTION_FAILED;
-        }
-
-        return ERROR_SUCCESS;
+        return remote_SetFeatureState(remote, szFeature, iState);
     }
 
     rc = MSI_SetFeatureStateW(package,szFeature,iState);
@@ -1128,39 +946,22 @@ UINT WINAPI MsiGetFeatureStateW(MSIHANDLE hInstall, LPCWSTR szFeature,
 
     TRACE("%d %s %p %p\n", hInstall, debugstr_w(szFeature), piInstalled, piAction);
 
+    if (!szFeature)
+        return ERROR_UNKNOWN_FEATURE;
+
     package = msihandle2msiinfo(hInstall, MSIHANDLETYPE_PACKAGE);
     if (!package)
     {
-        HRESULT hr;
-        BSTR feature;
-        IWineMsiRemotePackage *remote_package;
+        MSIHANDLE remote;
 
-        remote_package = (IWineMsiRemotePackage *)msi_get_remote(hInstall);
-        if (!remote_package)
+        if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        feature = SysAllocString(szFeature);
-        if (!feature)
-        {
-            IWineMsiRemotePackage_Release(remote_package);
-            return ERROR_OUTOFMEMORY;
-        }
+        /* FIXME: should use SEH */
+        if (!piInstalled || !piAction)
+            return RPC_X_NULL_REF_POINTER;
 
-        hr = IWineMsiRemotePackage_GetFeatureState(remote_package, feature,
-                                                   piInstalled, piAction);
-
-        SysFreeString(feature);
-        IWineMsiRemotePackage_Release(remote_package);
-
-        if (FAILED(hr))
-        {
-            if (HRESULT_FACILITY(hr) == FACILITY_WIN32)
-                return HRESULT_CODE(hr);
-
-            return ERROR_FUNCTION_FAILED;
-        }
-
-        return ERROR_SUCCESS;
+        return remote_GetFeatureState(remote, szFeature, piInstalled, piAction);
     }
 
     ret = MSI_GetFeatureStateW(package, szFeature, piInstalled, piAction);
@@ -1260,39 +1061,28 @@ UINT WINAPI MsiGetFeatureCostW(MSIHANDLE hInstall, LPCWSTR szFeature,
     TRACE("(%d %s %i %i %p)\n", hInstall, debugstr_w(szFeature),
           iCostTree, iState, piCost);
 
+    if (!szFeature)
+        return ERROR_INVALID_PARAMETER;
+
     package = msihandle2msiinfo(hInstall, MSIHANDLETYPE_PACKAGE);
     if (!package)
     {
-        HRESULT hr;
-        BSTR feature;
-        IWineMsiRemotePackage *remote_package;
+        MSIHANDLE remote;
 
-        remote_package = (IWineMsiRemotePackage *)msi_get_remote(hInstall);
-        if (!remote_package)
+        if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        feature = SysAllocString(szFeature);
-        if (!feature)
-        {
-            IWineMsiRemotePackage_Release(remote_package);
-            return ERROR_OUTOFMEMORY;
-        }
+        /* FIXME: should use SEH */
+        if (!piCost)
+            return RPC_X_NULL_REF_POINTER;
 
-        hr = IWineMsiRemotePackage_GetFeatureCost(remote_package, feature,
-                                                  iCostTree, iState, piCost);
+        return remote_GetFeatureCost(remote, szFeature, iCostTree, iState, piCost);
+    }
 
-        SysFreeString(feature);
-        IWineMsiRemotePackage_Release(remote_package);
-
-        if (FAILED(hr))
-        {
-            if (HRESULT_FACILITY(hr) == FACILITY_WIN32)
-                return HRESULT_CODE(hr);
-
-            return ERROR_FUNCTION_FAILED;
-        }
-
-        return ERROR_SUCCESS;
+    if (!piCost)
+    {
+        msiobj_release( &package->hdr );
+        return ERROR_INVALID_PARAMETER;
     }
 
     feature = msi_get_loaded_feature(package, szFeature);
@@ -1518,38 +1308,18 @@ UINT WINAPI MsiSetComponentStateW(MSIHANDLE hInstall, LPCWSTR szComponent,
     MSIPACKAGE* package;
     UINT ret;
 
+    if (!szComponent)
+        return ERROR_UNKNOWN_COMPONENT;
+
     package = msihandle2msiinfo(hInstall, MSIHANDLETYPE_PACKAGE);
     if (!package)
     {
-        HRESULT hr;
-        BSTR component;
-        IWineMsiRemotePackage *remote_package;
+        MSIHANDLE remote;
 
-        remote_package = (IWineMsiRemotePackage *)msi_get_remote(hInstall);
-        if (!remote_package)
+        if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        component = SysAllocString(szComponent);
-        if (!component)
-        {
-            IWineMsiRemotePackage_Release(remote_package);
-            return ERROR_OUTOFMEMORY;
-        }
-
-        hr = IWineMsiRemotePackage_SetComponentState(remote_package, component, iState);
-
-        SysFreeString(component);
-        IWineMsiRemotePackage_Release(remote_package);
-
-        if (FAILED(hr))
-        {
-            if (HRESULT_FACILITY(hr) == FACILITY_WIN32)
-                return HRESULT_CODE(hr);
-
-            return ERROR_FUNCTION_FAILED;
-        }
-
-        return ERROR_SUCCESS;
+        return remote_SetComponentState(remote, szComponent, iState);
     }
 
     ret = MSI_SetComponentStateW(package, szComponent, iState);
@@ -1569,39 +1339,22 @@ UINT WINAPI MsiGetComponentStateW(MSIHANDLE hInstall, LPCWSTR szComponent,
     TRACE("%d %s %p %p\n", hInstall, debugstr_w(szComponent),
            piInstalled, piAction);
 
+    if (!szComponent)
+        return ERROR_UNKNOWN_COMPONENT;
+
     package = msihandle2msiinfo(hInstall, MSIHANDLETYPE_PACKAGE);
     if (!package)
     {
-        HRESULT hr;
-        BSTR component;
-        IWineMsiRemotePackage *remote_package;
+        MSIHANDLE remote;
 
-        remote_package = (IWineMsiRemotePackage *)msi_get_remote(hInstall);
-        if (!remote_package)
+        if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        component = SysAllocString(szComponent);
-        if (!component)
-        {
-            IWineMsiRemotePackage_Release(remote_package);
-            return ERROR_OUTOFMEMORY;
-        }
+        /* FIXME: should use SEH */
+        if (!piInstalled || !piAction)
+            return RPC_X_NULL_REF_POINTER;
 
-        hr = IWineMsiRemotePackage_GetComponentState(remote_package, component,
-                                                     piInstalled, piAction);
-
-        SysFreeString(component);
-        IWineMsiRemotePackage_Release(remote_package);
-
-        if (FAILED(hr))
-        {
-            if (HRESULT_FACILITY(hr) == FACILITY_WIN32)
-                return HRESULT_CODE(hr);
-
-            return ERROR_FUNCTION_FAILED;
-        }
-
-        return ERROR_SUCCESS;
+        return remote_GetComponentState(remote, szComponent, piInstalled, piAction);
     }
 
     ret = MSI_GetComponentStateW( package, szComponent, piInstalled, piAction);
@@ -1620,20 +1373,12 @@ LANGID WINAPI MsiGetLanguage(MSIHANDLE hInstall)
     package = msihandle2msiinfo(hInstall, MSIHANDLETYPE_PACKAGE);
     if (!package)
     {
-        HRESULT hr;
-        LANGID lang;
-        IWineMsiRemotePackage *remote_package;
+        MSIHANDLE remote;
 
-        remote_package = (IWineMsiRemotePackage *)msi_get_remote(hInstall);
-        if (!remote_package)
+        if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        hr = IWineMsiRemotePackage_GetLanguage(remote_package, &lang);
-
-        if (SUCCEEDED(hr))
-            return lang;
-
-        return 0;
+        return remote_GetLanguage(remote);
     }
 
     langid = msi_get_property_int( package->db, szProductLanguage, 0 );
@@ -1677,26 +1422,12 @@ UINT WINAPI MsiSetInstallLevel(MSIHANDLE hInstall, int iInstallLevel)
     package = msihandle2msiinfo(hInstall, MSIHANDLETYPE_PACKAGE);
     if (!package)
     {
-        HRESULT hr;
-        IWineMsiRemotePackage *remote_package;
+        MSIHANDLE remote;
 
-        remote_package = (IWineMsiRemotePackage *)msi_get_remote(hInstall);
-        if (!remote_package)
+        if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        hr = IWineMsiRemotePackage_SetInstallLevel(remote_package, iInstallLevel);
-
-        IWineMsiRemotePackage_Release(remote_package);
-
-        if (FAILED(hr))
-        {
-            if (HRESULT_FACILITY(hr) == FACILITY_WIN32)
-                return HRESULT_CODE(hr);
-
-            return ERROR_FUNCTION_FAILED;
-        }
-
-        return ERROR_SUCCESS;
+        return remote_SetInstallLevel(remote, iInstallLevel);
     }
 
     r = MSI_SetInstallLevel( package, iInstallLevel );
