@@ -445,3 +445,42 @@ ULONG WINAPI EtwRegisterTraceGuidsA( WMIDPREQUEST RequestAddress,
           debugstr_a(MofResourceName), RegistrationHandle);
     return ERROR_SUCCESS;
 }
+
+/******************************************************************************
+ *                  EtwUnregisterTraceGuids (NTDLL.@)
+ */
+ULONG WINAPI EtwUnregisterTraceGuids( TRACEHANDLE RegistrationHandle )
+{
+    if (!RegistrationHandle)
+         return ERROR_INVALID_PARAMETER;
+
+    FIXME("%s: stub\n", wine_dbgstr_longlong(RegistrationHandle));
+    return ERROR_SUCCESS;
+}
+
+/******************************************************************************
+ *                  EtwEventEnabled (NTDLL.@)
+ */
+BOOLEAN WINAPI EtwEventEnabled( REGHANDLE handle, const EVENT_DESCRIPTOR *descriptor )
+{
+    FIXME("(%s, %p): stub\n", wine_dbgstr_longlong(handle), descriptor);
+    return FALSE;
+}
+
+/******************************************************************************
+ *                  EtwEventWrite (NTDLL.@)
+ */
+ULONG WINAPI EtwEventWrite( REGHANDLE handle, const EVENT_DESCRIPTOR *descriptor, ULONG count,
+    EVENT_DATA_DESCRIPTOR *data )
+{
+    FIXME("(%s, %p, %u, %p): stub\n", wine_dbgstr_longlong(handle), descriptor, count, data);
+    return ERROR_SUCCESS;
+}
+
+/***********************************************************************
+ *		    DbgUiRemoteBreakin (NTDLL.@)
+ */
+void WINAPI DbgUiRemoteBreakin( void *arg )
+{
+    FIXME("stub\n");
+}
